<?php

class fields_boxes extends fields_base
{
  function fields_boxes()
  {
  }

  function think()
  {
    parent::think();
    $this->load();
    global $action;
    switch ($action)
    {
      case ACT_ADD_BOX:
        $this->add_box();
        break;
      case ACT_EDIT_BOX:
        $this->edit_box();
        break;
      case ACT_DELETE_BOX:
        $this->delete_box();
        break;
      default:
        add_action('fields_show', array($this, 'show_index'));
        break;
    }
  }
    
  function get_title($title)
  {
    global $action;
    if ($action == ACT_EDIT_BOX)
    {
      return 'Fields - Edit box';
    }
    else
    {
      return 'Fields - Boxes';
    }
  }
  
  /* actions */
  function get($item_key)
  {
    global $boxes;
    if (is_array($boxes))
    {
      foreach ($boxes as $key => $box)
      {
        if (strcasecmp($item_key, $box['box_key']) == 0)
        {
          return $box;
        }
      }
    }
    return false;
  }
  
  function load()
  {
    global $boxes;
    if (!is_array($groups))
      $boxes = get_option(OPTION_BOXES);
    if (!is_array($boxes))
      $boxes = array();
    return $boxes;
  }
  
  function update()
  {
    global $boxes;
    update_option(OPTION_BOXES, $boxes);
  }  
  
  // shouldn't be called out of class scope
  function add_box()
  {
    global $boxes;
    $box_key = esc_attr($_POST['box-key']);
    
    if ((isset($box_key)) && ($box_key != ''))
    {
      // make sure it does not exist
      if (!$this->get($box_key))
      {        
        $box = array();
        $box['box_key'] = $box_key;
        $box['title'] = esc_attr($_POST['title']);
        $box['description'] = esc_attr($_POST['description']);
        
        // included groups
        $groups = $_POST['groups'];
        if (is_array($groups))
        {
          foreach ($groups as $g)
            $box['groups'][$g] = esc_attr($g);
        }
        
        // supported post types
        $post_types = $_POST['post_types'];
        if (is_array($post_types))
        {
          foreach ($post_types as $g)
            $box['post_types'][$g] = esc_attr($g);
        }
        
        $boxes[$box_key] = $box;
        $this->update();
        //$this->add_update("$box_key added.");
      }
      else
        $this->add_error("$box_key already exists");
    }
    else
      $this->add_error("Box key cannot be blank");
        
    add_action('fields_show', array($this, 'show_index'));
  }
  
  function edit_box()
  {
    global $boxes, $form_method;
    $item = esc_attr($_REQUEST['item']);
    
    // item not right
    if ((!isset($item)) || ($item == ''))
    {
      $this->add_error("Item is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }
    
    // edit submitted
    if ($form_method == POST)
    {
      $box_key = esc_attr($_POST['box-key']);
      
      if ((!isset($box_key)) || ($box_key == ''))
      {
        global $box;
        $box = $this->get($item);
        $this->add_error("Box key cannot be blank");
        add_action('fields_show', array($this, 'show_edit'));
        return;
      }
      
      // new key must not already exist
      if (strcasecmp($item, $box_key) != 0)
      {
        if ($this->get($box_key))
        {
          global $box;
          $box = $this->get($box_key);
          $this->add_error("$box_key already exist");
          add_action('fields_show', array($this, 'show_edit'));
          return;
        }
      }
      
      $box['box_key'] = $box_key;
      $box['title'] = esc_attr($_POST['title']);
      $box['description'] = esc_attr($_POST['description']);
      
      // included groups
      $groups = $_POST['groups'];
      if (is_array($groups))
      {
        foreach ($groups as $g)
          $box['groups'][$g] = esc_attr($g);
      }
      
      // supported post types
      $post_types = $_POST['post_types'];
      if (is_array($post_types))
      {
        foreach ($post_types as $g)
          $box['post_types'][$g] = esc_attr($g);
      }
      
      unset($boxes[$item]);
      $boxes[$box_key] = $box;
      $this->update();
      //$this->add_update("$box_key updated.");
      
      add_action('fields_show', array($this, 'show_index'));
    }
    // show edit form
    elseif ($form_method = GET)
    {
      global $box;
      $box = $this->get($item);
      add_action('fields_show', array($this, 'show_edit'));
    }
    // ???
    else
    {
      $this->add_error("Unknown error while editing box");
      add_action('fields_show', array($this, 'show_index'));
    }   
  }
  
  function list_groups($arr, $label_class) {
    $f = new fields_groups();
    $i = 1;
    $groups = $f->load();
    foreach ($groups as $key => $group) {
      $checked = false;
      if (is_array($arr))
      foreach ($arr as $ak => $v)
        if (strcasecmp($ak, $group['group_key']) == 0) {
          $checked = true;
          break;
        }
      if ($checked)
        $checked = ' checked="checked"';
      else
        $checked = '';
      $title = $group['title'];
      echo "<div class='check-row'><input type='checkbox' name='groups[]' id='group-$i' value='$key'$checked></input><label class='input-label' for='group-$i' class='$label_class'>$title</label></div>";
      $i++;
    }
    echo '<div class"cut"></div>';
  }
  
  function list_post_types($arr, $label_class) {
    $args=array(
      'show_ui' => true
    );
    $post_types=get_post_types($args);
    $i = 1;
    foreach ($post_types as $key => $post_type) {
      $checked = false;
      if (is_array($arr))
      foreach ($arr as $ak => $v) {
        if (strcasecmp($ak, $post_type) == 0) {
          $checked = true;
          break;
        }
      }
      if ($checked)
        $checked = ' checked="checked"';
      else
        $checked = '';
      echo "<div class='check-row'><input type='checkbox' name='post_types[]' id='post-type-$i'".
           " value='$post_type'$checked></input><label class='input-label' for='post-type-$i' class='$label_class'>".
           "$post_type</label></div>";
      $i++;
    }
    echo '<div class"cut"></div>';
  }  
  
  function delete_box()
  {
    global $boxes;
    $item = esc_attr($_REQUEST['item']);
    if ((isset($item)) && ($item != ''))
    {
      if ($this->get($item))
      {
        unset($boxes[$item]);
        $this->update();
        $this->add_update("$item deleted.");
      }
      else
        $this->add_error("$item not found.");
    }

    add_action('fields_show', array($this, 'show_index'));
  }
  
  /* show: index */  
  function show_index()
  {?>
    <div id="col-container">
      <?php 
      $this->show_index_left();
      $this->show_index_right();
      ?>
    </div>
  <?php
  }
  
  function show_index_left()
  {?>
    <div id="col-right">
      <div class="col-wrap">
      <table class="widefat need-pad">
        <thead>
          <?php $this->show_table_header(); ?>
        </thead>
        
        <tbody>
          <?php $this->show_boxes(); ?>
        </tbody>
        
        <tfoot>
          <?php $this->show_table_header(); ?>
        </tfoot>
      </table>
      <p>Boxes can include groups and support a number of selected post types</p>
      </div>
    </div>
  <?php
  }
  
  function show_table_header()
  {?>
  	<tr>
    	<th><?php _e('Title', 'fields'); ?></th>
    	<th><?php _e('Groups', 'fields'); ?></th>
    	<th><?php _e('Post types', 'fields'); ?></th>
    	<th><?php _e('Actions', 'fields'); ?></th>
  	</tr>
  <?php
  }
  
  // list the boxes on index page
  function show_boxes()
  {
    global $boxes, $fields_groups;
    $i = 1;
    if (is_array($boxes))
    {
      global $page_url, $tab;
      $url = $page_url . "&tab=$tab";
      foreach ($boxes as $key => $box)
      {
        if ($i == 1)
          $class = "alternate";
        else
          $class = "";
        $i = 3 - $i;
        $box_url = $url . "&item=$key";
        $group_names = array();
        if (is_array($box['groups']))
        {
          foreach ($box['groups'] as $group_key)
          {
            $group = $fields_groups->get($group_key);
            if (is_array($group))
              array_push($group_names, $group['title']);
          }
        }
        $group_names = implode(', ', $group_names);
        if (is_array($box['post_types']))
          $post_types = implode(', ', $box['post_types']);
        $edit_url = $box_url . '&action=' . ACT_EDIT_BOX;
        echo "<tr class='$class'>". 
             "<td><a class='title' href='$edit_url'>" . $box['title'] . '</a></td>' .
             "<td>$group_names</td>" .
             "<td>$post_types</td>" .
             "<td><a href='$edit_url'>Edit</a> | " .
             "<a href='$box_url&action=" . ACT_DELETE_BOX . "'>Delete</a></td>" .
             '</tr>';
      }
    }
  }  
  
  function show_index_right()
  {
    ?>
    <!-- add box form -->      
    <div id="col-left">
      <div class="col-wrap">
      <div class="form-wrap">
        <h3>Add new box</h3>
        <form method="post" action=<?php global $page_url; echo $page_url; ?>>
          <?php settings_fields(''); ?>
          <input type="hidden" id="tab" name="tab" value="boxes"></input>
      		<input type="hidden" id="action" name="action" value="<?php echo ACT_ADD_BOX; ?>"></input>
          <div class="form-field form-required">
          	<label for="box-key">Key</label>
          	<input type="text" aria-required="true" value="" id="tag-box-key" name="box-key"></input>
          	<p>Keys should be in lowercase and contain only alphabet, numbers and dashes '-'.</p>
          </div>        		
    
          <div class="form-field form-required">
          	<label for="box-title">Title</label>
          	<input type="text" aria-required="true" value="" id="tag-title" name="title"></input>
          </div>        		

          <div class="form-field">
          	<label for="box-description">Description</label>
          	<textarea rows="5" id="tag-description" name="description"></textarea>
          	<p></p>
          </div>            

          <div class="form-field">
          	<label for="box-groups">Included groups</label>
          	<?php $this->list_groups(0, 'normal'); ?>
          	<p>Select which groups are included in your box</p>
          </div>            
    
          <div class="form-field">
          	<label for="box-groups">Supported post types</label>
          	<?php $this->list_post_types(0, 'normal'); ?>
          	<p>Select which post types are supported by your box</p>
          </div>            
      			    
      		<p><input class="button-primary" type="submit" value="<?php _e('Save' ,'fields'); ?>"></input></p>
      	</form>
      </div><!-- form-wrap -->
      </div><!-- col-wrap -->
    </div><!-- col-left -->
    <?php  
  }
  
  /* show: edit */
  function show_edit()
  {
    global $page_url, $box;
    ?>
    <form method="post" action="<?php echo $page_url; ?>">
      <?php settings_fields(''); ?>
      <input type="hidden" id='tab' name='tab' value='boxes'></input>
      <input type="hidden" id='action' name='action' value='<?php echo ACT_EDIT_BOX; ?>'></input>
      <input type="hidden" id='item' name='item' value='<?php echo $box['box_key']; ?>'></input>
      
      <table class="form-table">
        <tbody>
        
        <tr>
    			<th valign="top" scope="row"><label for="tag-box-key">Key</label></th>
    			<td><input type="text" aria-required="true" size="40" value="<?php echo $box['box_key'] ?>" id="tag-box-key" name="box-key"></input>
    			<p class="description">Keys should be in lowercase and contain only alphabet, numbers and dashes '-'.</p></td>
    		</tr> 
    		
        <tr>
    			<th valign="top" scope="row"><label for="tag-box-title">Title</label></th>
    			<td><input type="text" aria-required="true" size="40" value="<?php echo $box['title']; ?>" id="tag-box-title" name="title"></input>
          </td>
    		</tr> 
    		
        <tr>
    			<th valign="top" scope="row"><label for="description">Description</label></th>
    			<td><textarea style="width: 50%;" rows="5" id="description" name="description"><?php echo $box['description']; ?></textarea>
    		</tr>    		
    		        
        <tr>
    			<th valign="top" scope="row"><label for="groups">Included groups</label></th>
    			<td><?php $this->list_groups($box['groups'], ''); ?>    			
    			<p class="description">Select which groups are included in your box</p></td>
    		</tr>      		

        <tr>
    			<th valign="top" scope="row"><label for="post_types">Supported post types</label></th>
    			<td><?php $this->list_post_types($box['post_types'], ''); ?>    			
    			<p class="description">Select which post types are supported by your box</p></td>
    		</tr> 
    		    		
        </tbody>
      </table>
      <p><input class="button-primary" type="submit" value="<?php _e('Update' ,'fields'); ?>"></input></p>
    </form>
    <?php
  }
}

?>