<?php

class fields_groups extends fields_base
{
  function fields_groups()
  {    
  }
  
  function think()
  {
    parent::think();
    $this->load();
    global $action;
    switch ($action)
    {
      case ACT_ADD_GROUP:
        $this->add_group();
        break;
      case ACT_EDIT_GROUP:
        $this->edit_group();
        break;
      case ACT_DELETE_GROUP:
        $this->delete_group();
        break;
      case ACT_ADD_FIELD:
        $this->add_field();
        break;
      case ACT_EDIT_FIELD:
        $this->edit_field();
        break;
      case ACT_DELETE_FIELD:
        $this->delete_field();
        break;
      default:
        add_action('fields_show', array($this, 'show_index'));
        break;
    }
  }
  
  function get_title($title)
  {
    global $action;
    switch ($action)
    {
      case ACT_EDIT_GROUP:
        return 'Fields - Edit group';
      case ACT_EDIT_FIELD:
        return 'Fields - Edit field';
      default:
        return 'Fields - Groups';
    }
  }  
  
  /* actions */
  function get($item_key)
  {
    global $groups;
    if (is_array($groups))
    {
      foreach ($groups as $key => $group)
      {
        if (strcasecmp($item_key, $group['group_key']) == 0)
        {
          return $group;
        }
      }
    }
    return false;
  }
  
  function load()
  {
    global $groups;
    if (!is_array($groups))
      $groups = get_option(OPTION_GROUPS);
    if (!is_array($groups))
      $groups = array();
    return $groups;
  }
  
  function update()
  {
    global $groups;
    sort_by_order(&$groups);
    update_option(OPTION_GROUPS, $groups);
  }
  
  // shouldn't be called out of class scope
  function add_group()
  {
    global $groups;
    $group_key = sanitize_title($_POST['group-key']);
    
    if ((isset($group_key)) && ($group_key != ''))
    {
      // make sure it does not exist
      if (!$this->get($group_key))
      {        
        $group = array();
        $group['group_key'] = $group_key;
        $group['title'] = esc_attr($_POST['title']);
        $group['order'] = esc_attr($_POST['order']);
        $group['description'] = esc_attr($_POST['description']);
                
        $groups[$group_key] = $group;
        $this->update();
        //$this->add_update("$group_key added.");
      }
      else
        $this->add_error("$group_key already exists");
    }
    else
      $this->add_error("Group key cannot be blank");
        
    add_action('fields_show', array($this, 'show_index'));
  }  
  
  function edit_group()
  {
    global $groups, $form_method;
    $item = sanitize_title($_REQUEST['item']);
    
    // item not right
    if ((!isset($item)) || ($item == ''))
    {
      $this->add_error("Item is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }
    
    // edit submitted
    if ($form_method == POST)
    {
      $group_key = esc_attr($_POST['group-key']);
      
      if ((!isset($group_key)) || ($group_key == ''))
      {
        global $group;
        $group = $this->get($item);
        $this->add_error("Group key cannot be blank");
        add_action('fields_show', array($this, 'show_edit'));
        return;
      }
      
      // new key must not already exist
      if (strcasecmp($item, $group_key) != 0)
      {
        if ($this->get($group_key))
        {
          global $group;
          $group = $this->get($item);
          $this->add_error("$group_key already exist");
          add_action('fields_show', array($this, 'show_edit'));
          return;
        }
      }
      
      $group['group_key'] = $group_key;
      $group['title'] = esc_attr($_POST['title']);
      $group['order'] = esc_attr($_POST['order']);
      $group['description'] = esc_attr($_POST['description']);
      $group['fields'] = $groups[$item]['fields'];
            
      unset($groups[$item]);
      $groups[$group_key] = $group;
      $this->update();
      //$this->add_update("$group_key updated.");
      
      add_action('fields_show', array($this, 'show_index'));
    }
    // show edit form
    elseif ($form_method = GET)
    {
      global $group;
      $group = $this->get($item);
      if (!is_array($group))
      {
        $this->add_error("Unknown error while editing group");
        add_action('fields_show', array($this, 'show_index'));
        return;
      }
      
      if (!isset($group['order']))
        $group['order'] = 0;
      add_action('fields_show', array($this, 'show_edit'));
    }
    // ???
    else
    {
      $this->add_error("Unknown error while editing group");
      add_action('fields_show', array($this, 'show_index'));
    }   
  }
  
  function delete_group()
  {
    global $groups;
    $item = esc_attr($_REQUEST['item']);
    if ((isset($item)) && ($item != ''))
    {
      if ($this->get($item))
      {
        unset($groups[$item]);
        $this->update();
        $this->add_update("$item deleted.");
      }
      else
        $this->add_error("$item not found.");
    }

    add_action('fields_show', array($this, 'show_index'));
  }
  
  function add_field()
  {
    global $groups, $form_method;
    $item = esc_attr($_POST['item']);
    if (($form_method == 'POST') && ($item != ''))
    {
      global $group;
      $group = $this->get($item);
      if (is_array($group))
      {
        $field_key = sanitize_title($_POST['field-key']);
        $in_group = in_groups($field_key, $groups);
        if (!$in_group)
        {
          $field = array();
          $field['field_key'] = $field_key;
          $field['title'] = esc_attr($_POST['title']);
          $field['type'] = esc_attr($_POST['type']);
          $field['note'] = esc_attr($_POST['note']);
          $field['order'] = esc_attr($_POST['order']);
          
          // additional          
          $field = apply_filters(FIELD_SAVE, $field, $group);
          
          $groups[$item]['fields'][$field_key] = $field;          
          sort_by_order(&$groups[$item]['fields']);
          
          $group = $this->get($item);
          $this->update();
        }
        else
        {
          $this->add_error("Field '$field_key' already exists in '$in_group'");
        }
        add_action('fields_show', array($this, 'show_edit'));
      }
    }
    else
    {
      $this->add_error("something");
      add_action('fields_show', array($this, 'show_index'));
    }
  }
  
  function get_field($group, $field_key)
  {
    if (is_array($group['fields']))
    {
      foreach ($group['fields'] as $key => $field)
      {
        if (strcasecmp($key, $field_key) == 0)
        {
          return $field;
        }
      }
    }
    return false;
  }
  
  function edit_field()
  {
    global $groups, $group, $form_method;
    $item = sanitize_title($_REQUEST['item']);
    $sub_item = sanitize_title($_REQUEST['sub_item']);
    
    // item not right
    if ((!isset($item)) || ($item == '') || (!($this->get($item))))
    {
      $this->add_error("Group is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }

    // sub item not right
    if ((!isset($sub_item)) || ($sub_item == ''))
    {
      $this->add_error("Field is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }
    
    $group = $this->get($item);
    
    // edit submitted
    if ($form_method == POST)
    {
      $field_key = sanitize_title($_POST['field-key']);
      
      if ((!isset($field_key)) || ($field_key == ''))
      {
        global $field;
        $field = $group[$sub_item];
        $this->add_error("Field key cannot be blank");
        add_action('fields_show', array($this, 'show_edit_field'));
        return;
      }
      
      // new key must not already exist
      if (strcasecmp($sub_item, $field_key) != 0)
      {
        $in_group = in_groups($field_key, $groups);
        if ($in_group)
        {
          global $field;
          $field = $group[$sub_item];
          $this->add_error("Field '$field_key' already exist in '$in_group'");
          add_action('fields_show', array($this, 'show_edit_field'));
          return;
        }
      }
      
      // update field
      $field = array();
      $field['field_key'] = $field_key;
      $field['title'] = esc_attr($_POST['title']);
      $field['type'] = esc_attr($_POST['type']);
      $field['note'] = esc_attr($_POST['note']);
      $field['order'] = esc_attr($_POST['order']);
      
      // additional            
      $field = apply_filters(FIELD_SAVE, $field, $group);

      unset($groups[$item]['fields'][$sub_item]);
      $groups[$item]['fields'][$field_key] = $field;
      sort_by_order(&$groups[$item]['fields']);
      
      $this->update();
      $group = $groups[$item];
      //$this->add_update("$group_key updated.");
      
      add_action('fields_show', array($this, 'show_edit'));
    }
    // show edit form
    elseif ($form_method = GET)
    {
      global $field;
      $field = $this->get_field($group, $sub_item);
      if (!is_array($field))
      {
        $field = $group[$sub_item];
        $this->add_error("Unknown error while editing field");
        add_action('fields_show', array($this, 'show_edit'));
        return;
      }
      
      if (!isset($field['order']))
        $field['order'] = 0;
      add_action('fields_show', array($this, 'show_edit_field'));
    }
    // ???
    else
    {
      $this->add_error("Unknown error while editing field");
      add_action('fields_show', array($this, 'show_index'));
    }    
  }
  
  function delete_field()
  {
    global $groups;
    $item = esc_attr($_REQUEST['item']);
    $sub_item = esc_attr($_REQUEST['sub_item']);
    
    // item not right
    if ((!isset($item)) || ($item == '') || (!($this->get($item))))
    {
      $this->add_error("Group is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }

    // sub item not right
    if ((!isset($sub_item)) || ($sub_item == ''))
    {
      $this->add_error("Field is not valid for editing.");
      add_action('fields_show', array($this, 'show_index'));
      return;
    }
    
    global $group, $field;
    $group = $this->get($item);
    $field = $this->get_field($group, $sub_item);
    
    if (is_array($field))
    {
      unset($groups[$item]['fields'][$sub_item]);
      $this->update();
      $this->add_update("$sub_item deleted.");
      $group = $this->get($item);
      add_action('fields_show', array($this, 'show_edit'));      
    }
    else
    {
      $this->add_error("Field $sub_item not found");
      add_action('fields_show', array($this, 'show_edit'));
    }
  }
  
  // show: index
  function show_index()
  {?>
    <div id="col-container">
      <?php 
      $this->show_index_left();
      $this->show_index_right();
      ?>
    </div>
  <?php
  }
  
  function show_index_left()
  {?>
    <div id="col-right">
      <div class="col-wrap">
      <table class="widefat need-pad">
        <thead>
          <?php $this->show_table_header(); ?>
        </thead>
        
        <tbody>
          <?php $this->show_groups(); ?>
        </tbody>
        
        <tfoot>
          <?php $this->show_table_header(); ?>
        </tfoot>
      </table>
      <p>Groups are sets of fields that can be included in boxes. Groups are displayed as tabs in boxes.</p>
      </div>
    </div>
  <?php
  }
  
  function show_table_header()
  {?>
  	<tr>
    	<th><?php _e('Title', 'fields'); ?></th>
    	<th><?php _e('Fields', 'fields'); ?></th>
    	<th><?php _e('Order', 'fields'); ?></th>
    	<th><?php _e('Actions', 'fields'); ?></th>
  	</tr>
  <?php
  }
  
  // list the groups on index page
  function show_groups()
  {
    global $groups;
    if (is_array($groups))
    {      
      global $page_url, $tab;
      $i = 1;
      foreach ($groups as $key => $group)
      {
        if ($i == 1)
          $class = "alternate";
        else
          $class = "";
        $i = 3 - $i;      
        $fields = $this->get_fields_str($group['fields']);
        $edit_url = build_group_url($group, ACT_EDIT_GROUP);
        $delete_url = build_group_url($group, ACT_DELETE_GROUP);
        echo "<tr class='$class'>". 
             "<td><a class='title' href='$edit_url'>" . $group['title'] . '</a></td>' .
             "<td>$fields</td>" .
             '<td>' . $group['order'] . '</td>' .
             "<td><a href='$edit_url'>Edit</a> | " .
             "<a href='$delete_url'>Delete</a></td>" .
             '</tr>';
      }
    }
  }    
  
  function get_fields_str($fields)
  {
    $s = '';
    if (is_array($fields))
    {
      foreach ($fields as $key => $field)
      {
        $s .= ", " . $field['title'];
      }
    }
    else
      return '';
    return substr($s, 1);
  }  
  
  //
  function show_index_right()
  {
    ?>
    <!-- add group form -->      
    <div id="col-left">
      <div class="col-wrap">
      <div class="form-wrap">
        <h3>Add new group</h3>
        <form method="post" action=<?php global $page_url; echo $page_url; ?>>
          <?php settings_fields(''); ?>
          <input type="hidden" id="tab" name="tab" value="groups"></input>
      		<input type="hidden" id="action" name="action" value="<?php echo ACT_ADD_GROUP; ?>"></input>
          <div class="form-field form-required">
          	<label for="group-key">Key</label>
          	<input type="text" aria-required="true" value="" id="tag-group-key" name="group-key"></input>
          	<p>Keys should be in lowercase and contain only alphabet, numbers and dashes '-'.</p>
          </div>        		
    
          <div class="form-field form-required">
          	<label for="group-title">Title</label>
          	<input type="text" aria-required="true" value="" id="tag-title" name="title"></input>
          </div>        		

          <div class="form-field form-required">
          	<label for="group-description">Description</label>
          	<textarea rows="5" id="tag-description" name="description"></textarea>
          	<p></p>
          </div>            
      			    
          <div class="form-field form-required">
          	<label for="group-order">Order</label>
          	<input type="text" aria-required="true" value="0" id="tag-order" name="order"></input>
          	<p>Smaller values stay more on top of the group list in a box</p>
          </div>        		

      		<p><input class="button-primary" type="submit" value="<?php _e('Add New Group' ,'fields'); ?>"></input></p>
      	</form>
      </div><!-- form-wrap -->
      </div><!-- col-wrap -->
    </div><!-- col-left -->
    <?php  
  }
  
  /* show: edit */
  function show_edit()
  {
    global $page_url, $group;
    ?>
    <form method="post" action="<?php echo $page_url; ?>">
      <?php settings_fields(''); ?>
      <input type="hidden" id='tab' name='tab' value='groups'></input>
      <input type="hidden" id='action' name='action' value='<?php echo ACT_EDIT_GROUP; ?>'></input>
      <input type="hidden" id='item' name='item' value='<?php echo $group['group_key']; ?>'></input>
      
      <table class="form-table">
        <tbody>
        
        <tr>
    			<th valign="top" scope="row"><label for="tag-group-key">Key</label></th>
    			<td><input type="text" aria-required="true" size="40" value="<?php echo $group['group_key'] ?>" id="tag-group-key" name="group-key"></input>
    			<p class="description">Keys should be in lowercase and contain only alphabet, numbers and dashes '-'.</p></td>
    		</tr> 
    		
        <tr>
    			<th valign="top" scope="row"><label for="tag-group-title">Title</label></th>
    			<td><input type="text" aria-required="true" size="40" value="<?php echo $group['title']; ?>" id="tag-group-title" name="title"></input>
          </td>
    		</tr> 
    		
        <tr>
    			<th valign="top" scope="row"><label for="tag-group-order">Order</label></th>
    			<td><input type="text" aria-required="true" size="40" value="<?php echo $group['order']; ?>" id="tag-group-order" name="order"></input>
          </td>
    		</tr> 

        <tr>
    			<th valign="top" scope="row"><label for="description">Description</label></th>
    			<td><textarea style="width: 50%;" rows="5" id="description" name="description"><?php echo $group['description']; ?></textarea>
    		</tr>    		
    		        
    		    		
        </tbody>
      </table>
      <p><input class="button-primary" type="submit" value="<?php _e('Update' ,'fields'); ?>"></input></p>
    </form>
    <?php
    $this->show_group_fields($group);
  }
  
  /* show: fields */
  function show_group_fields($group)
  {?>
    <div id="col-container">
      <?php 
      $this->show_group_fields_left($group);
      $this->show_group_fields_right($group);
      ?>
    </div>
  <?php
  }
  
  function show_group_fields_left($group)
  {?>
    <div id="col-right">
    <div class="col-wrap">
    <div class="form-wrap"> 
      <table class="widefat need-pad">
      <thead>
        <?php $this->show_fields_header(); ?>
      </thead>
      <tbody>
        <?php $this->show_fields($group); ?>
      </tbody>
      <tfoot>
        <?php $this->show_fields_header(); ?>
      </tfoot>
      </table>
    </div><!-- form-warp -->
    </div><!-- col-wrap -->
    </div><!-- col-right -->
  <?php
  }
  
  function show_fields_header()
  {?>
  	<tr>
    	<th><?php _e('Title', 'fields'); ?></th>
    	<th><?php _e('Type', 'fields'); ?></th>
    	<th><?php _e('Order', 'fields'); ?></th>
    	<th><?php _e('Actions', 'fields'); ?></th>
  	</tr>
  <?php
  }
  
  function show_fields($group)
  {
    if ((is_array($group)) && (is_array($group['fields'])))
    {
      global $page_url, $calce_field_types, $tab;
      $url = $page_url . "&tab=$tab";
      foreach ($group['fields'] as $key => $field) 
      {
        $item_url = $url . "&item=" . $group['group_key'] . "&sub_item=" . $key;
        $edit_url = $item_url . '&action=' . ACT_EDIT_FIELD;
        $delete_url = $item_url . '&action=' . ACT_DELETE_FIELD;
        $title = $field['title'];
        ?>
          <tr>
            <td><?php echo "<a class='title' href='$edit_url'>$title</a>"; ?></td>
            <td><?php echo $calce_field_types[$field['type']]['title']; ?></td>
            <td><?php echo $field['order']; ?></td>
            <td><a href="<?php echo $edit_url; ?>">Edit</a> | <a href="<?php echo $delete_url ?>">Delete</a></td>
          </tr>                
        <?php
      }
    }   
  }  
  
  // add-field form
  function show_group_fields_right($group)
  {
    global $page_url;
    ?>
    <div id="col-left">
    <div class="col-wrap">
    <div class="form-wrap">
      <h3>Add new field</h3>
      <form method="post" action=<?php global $page_url; echo $page_url; ?>>
        <input type="hidden" id="item" name="item" value="<?php echo $group['group_key']; ?>"></input>
        <input type="hidden" id="tab" name="tab" value="groups"></input>
    		<input type="hidden" id="action" name="action" value="<?php echo ACT_ADD_FIELD; ?>"></input>
    		
    		<div class="form-field form-required">
        	<label for="tag-field-key">Key</label>
        	<input type="text" aria-required="true" size="20" id="tag-field-key" name="field-key"></input>
        	<p>A field key must be unique and should be in lowercase and contain only alphabet, numbers and dashes '-'.</p>
    		</div>

    		<div class="form-field form-required">
        	<label for="tag-name">Title</label>
        	<input type="text" aria-required="true" id="tag-title" name="title"></input>
        	<p>As displayed in your box</p>
    		</div>

    		<div class="form-field form-required">
        	<label for="tag-note">Note</label>
        	<input type="text" aria-required="true" id="tag-note" name="note"></input>
        	<p>Note will be shown below the field</p>
    		</div>
    		
    		<div class="form-field form-required">
        	<label for="tag-field-key">Order</label>
        	<input type="text" aria-required="true" id="tag-order" name="order" value="0"></input>
    		</div>

    		<div class="form-field form-required">
        	<label for="tag-type">Type</label>
        	<?php $this->show_type_select(); ?>
    		</div>

        <!-- Additional -->
        <?php 
          $adds =  apply_filters(FIELD_SHOW, '', false); 
          echo $adds;
        ?> 		

    		<p class="submit"><input type="submit" value="<?php _e('Add New Field' ,'fields'); ?>"></input></p>
    		
      </form>      
    </div>
    </div>
    </div>        
  <?php 
  }
  
  function show_edit_field()
  {
    global $group, $field;
    ?>
    <form method="post" action="<?php echo $page_url; ?>">
      <?php settings_fields(''); ?>
      <input type="hidden" id='tab' name='tab' value='groups'></input>
      <input type="hidden" id='action' name='action' value='<?php echo ACT_EDIT_FIELD; ?>'></input>
      <input type="hidden" id='item' name='item' value='<?php echo $group['group_key']; ?>'></input>
      
      <table class="form-table">
      <tbody>
        <tr>
      		<th valign="top" scope="row"><label for="tag-group-key">Key</label></th>
      		<td><input type="text" aria-required="true" value="<?php echo $field['field_key'] ?>" id="tag-field-key" name="field-key"></input>
      		<p class="description">A field key must be unique and should be in lowercase and contain only alphabet, numbers and dashes '-'.</p></td>
      	</tr>
      	
        <tr>
      		<th valign="top" scope="row"><label for="tag-group-title">Title</label></th>
      		<td><input type="text" aria-required="true" value="<?php echo $field['title'] ?>" size="40" id="tag-title" name="title"></input></td>
      	</tr>

        <tr>
      		<th valign="top" scope="row"><label for="tag-group-title">Note</label></th>
      		<td><input type="text" aria-required="true" value="<?php echo $field['note'] ?>" size="40" id="tag-note" name="note"></input></td>
      	</tr>
      	
        <tr>
      		<th valign="top" scope="row"><label for="tag-order">Order</label></th>
      		<td><input type="text" aria-required="true"  value="<?php echo $field['order'] ?>" id="tag-order" name="order"></input></td>
      	</tr>
      	
        <tr>
      		<th valign="top" scope="row"><label for="tag-type">Type</label></th>
      		<td><?php $this->show_type_select($field['type']); ?></td>
      	</tr>
      	
      	<!-- Additional -->
        <?php 
          $adds =  apply_filters(FIELD_SHOW, '', $field); 
          echo $adds;
        ?>
      	
      </tbody>
      </table>
      
      <p class="submit"><input class="button-primary" type="submit" value="<?php _e('Update Field' ,'fields'); ?>"></input>
      <a class="back-link" href="<?php echo build_group_url($group, ACT_EDIT_GROUP); ?>">Back to <?php echo $group['title']; ?></a>
      </p>
    </form>
    <?php
  }
  
  function show_type_select($value = 'text')
  {
    global $calce_field_types;
    $script = '<script type="text/javascript">var calce_types=new Array();';
  	echo "<select aria-required='true' id='tag-type' name='type'>";
  	foreach ($calce_field_types as $key => $field_type)
  	{
      $title = $field_type['title'];
      $class = $field_type['class'];
      $script .= "calce_types['$key'] = '$class';";
      if ($key == $value)
       echo "<option selected value='$key'>$title</option>";
      else
       echo "<option value='$key'>$title</option>";
  	}
  	$script .= '</script>';
  	echo '</select>';
  	echo $script;
  }
}

?>